from Products.CMFCore.utils import getToolByName
from Products.CMFCore.DirectoryView import addDirectoryViews

from Products.CMFAggregator import cmfaggregator_globals
import string
from cStringIO import StringIO

out = StringIO()

def addTools(self):
    """ """
    portal_url = getToolByName(self, 'portal_url')

    # Add the CMFAggregator tool to the site's root
    p = portal_url.getPortalObject()    
    try:
        obj = p.manage_addProduct['CMFAggregator'].manage_addTool(type="CMF Aggregation Tool")
        out.write('Added aggregation tool to portal successfully \n')
    except:
        out.write('Could not add aggregation tool to portal.\n')


def addTypes(self):
    """ """
    type_tool = getToolByName(self, 'portal_types')
    try:
        type_tool.manage_addTypeInformation(add_meta_type = 'Factory-based Type Information', 
                                     id='RSSFeed', 
                                     typeinfo_name = 'CMFAggregator: RSSFeed')
        type_tool.manage_addTypeInformation(add_meta_type = 'Factory-based Type Information', 
                                     id='RSSStory', 
                                     typeinfo_name = 'CMFAggregator: RSSStory')
        type_tool.manage_addTypeInformation(add_meta_type = 'Factory-based Type Information', 
                                     id='RSSCollection', 
                                     typeinfo_name = 'CMFAggregator: RSSCollection')
        out.write('Added RSSFeed, RSSStory and RSSCollection into portal_types.\n')
    except: 
        out.write('Could not add new content types into portal_types.\n')


def addSkins(self):
    """ """
    skin_tool = getToolByName(self, 'portal_skins')
    try:
        addDirectoryViews( skin_tool, 'skins', cmfaggregator_globals)
        out.write('directory view cmfaggregator was successfully installed into portal_skins.\n')
    except:
        out.write('Could not install directory view cmfaggregator into portal_skins.\n')

    dirViews = ['cmfaggregator', ]
    for skin in skin_tool.getSkinSelections():
        path = skin_tool.getSkinPath(skin)
        path = map( string.strip, string.split( path,',' ) )
        
        for dirView in dirViews:
            if not dirView in path:
                try:
                    path.insert( path.index( 'custom' ) + 1, dirView)
                    out.write('%(dirView)s was successfully installed into %(skin)s.\n' %{'dirView':dirView,'skin':skin})
                except ValueError:
                    path.append(dirView)
                    out.write('%(dirView)s was successfully installed into %(skin)s.\n' %{'dirView':dirView,'skin':skin})
                except: out.write('Could not install %(dirView)s into %(skin)s.\n' %{'dirView':dirView,'skin':skin})
            else:
                out.write('%(dirView)s subskin was already installed into %(skin)s.\n' %{'dirView':dirView,'skin':skin})

        path = string.join( path, ', ' )
        skin_tool.addSkinSelection( skin, path )


def registerForms(self):
    """ """
    form_tool = getToolByName(self, 'portal_form')
    nav_tool = self.portal_navigation

    #Registering with Form Tool
    forms = {'rsscollection_edit_form':['validate_id','validate_rsscollection_edit'] ,
             'rssfeed_edit_form':['validate_id','validate_rssfeed_edit'] ,
             'rssstory_edit_form':['validate_id','validate_rssstory_edit']
            }
    for form in forms.keys():
        try: 
            form_tool.setValidators(form, forms[form])
        except: 
            out.write("Unable to register %s with portal form \n" %form)
        else: 
            out.write("%s was registered with portal form \n" %form)

    #Registering with Navigation Tool
    navs = ["default.rsscollection_edit_form.failure = rsscollection_edit_form",
            "default.rsscollection_edit_form.success = script:rsscollection_edit",
            "default.rsscollection_edit.failure = rsscollection_edit_form",
            "default.rsscollection_edit.success = action:view",

            "default.rssfeed_edit_form.failure = rssfeed_edit_form",
            "default.rssfeed_edit_form.success = script:rssfeed_edit",
            "default.rssfeed_edit.failure = rssfeed_edit_form",
            "default.rssfeed_edit.success = action:view", 

            "default.rssstory_edit_form.failure = rssstory_edit_form",
            "default.rssstory_edit_form.success = script:rssstory_edit",
            "default.rssstory_edit.failure = rssstory_edit_form",
            "default.rssstory_edit.success = action:view"
           ]

    import re
    re_transition = re.compile(r"\s*(?P<type>[^\.]*)\.(?P<page>[^\.]*)\.(?P<outcome>[^\s]*)\s*=\s*(?P<action>[^$]*)$")

    try:
        for nav in navs:
            nav = nav.strip()
            match = re_transition.match(nav)
            if match:
                nav_tool.addTransitionFor(match.group('type'), match.group('page'), match.group('outcome'), match.group('action'))
            else:
                out.write("Unable to parse '%s' in navigation properties list" % (nav))
    except: 
        out.write("Unable register with transition tool \n")
    else: 
        out.write("Registered with transition tool \n")



def install(self):
    """ """    
    out.write("=========== Add Aggregation Tool =================\n")
    addTools(self)

    out.write("=========== Add RSSType, RSSStory, RSSCollection type =================\n")
    addTypes(self)

    out.write("=========== Add Directory View and Skin path =================\n")
    addSkins(self)

    out.write("=========== Register form with portal_form and portal_navigation tool =================\n")
    registerForms(self)

    return out.getvalue()

