from Products.CMFDefault.SkinnedFolder import SkinnedFolder
from Globals import InitializeClass
from DateTime import DateTime
from Products.CMFCore.utils import getToolByName
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base

from md5 import md5
from binascii import b2a_base64
from AggregationTool import STATUS_DEAD
from Permissions import p_manageRSSCollection, p_View

#For debugging
from logger import logger
from AggregationTool import base64_to_id_trans, delete_chars

factory_type_information = ({'id': 'RSSCollection',
                             'content_icon': 'folder_icon.gif',
                             'meta_type': 'RSSCollection',
                             'description':""" """,
                             'product': 'CMFAggregator',
                             'factory': 'addRSSCollection',
                             'immediate_view': 'rsscollection_edit_form',
                             'allowed_content_types':['RSSFeed', 'RSSCollection'],
                             'actions':({'id': 'view',
                                          'name': 'View',
                                          'action': 'rsscollection_view',
                                          'permissions': (p_View,)},
                                         {'id': 'contents',
                                          'name': 'Contents',
                                          'action': 'folder_contents',
                                          'permissions': (p_manageRSSCollection,)},
                                         {'id': 'edit',
                                          'name': 'Edit',
                                          'action': 'portal_form/rsscollection_edit_form',
                                          'permissions': (p_manageRSSCollection,)},
                                         {'id': 'aggregate',
                                          'name': 'Aggregate',
                                          'action': 'aggregate',
                                          'permissions': (p_manageRSSCollection,)},
                                         {'id': 'import',
                                          'name': 'Import',
                                          'action': 'rsscollection_import_form',
                                          'permissions': (p_manageRSSCollection,)},
                                         {'id': 'schedule',
                                          'name': 'Schedule',
                                          'action': 'rsscollection_schedule_form',
                                          'permissions': (p_manageRSSCollection,)},
                                        ),
                             },
                            )

def addRSSCollection( self, id, title='', description=''):
    """ """
    o=RSSCollection( id=id, 
                     title=title, 
                     description=description
                    )
    self._setObject(id,o)

class RSSCollection(SkinnedFolder):
    """ """
    meta_type = 'RSSCollection'

    security = ClassSecurityInfo()

    def __init__(self, id, title='', description=''):
        """ """
        self.id = id
        self.title = title
        self.description = description

    def edit(self, id='', title='', description=''):
        """ """
        if id:
            self.id = id
        self.title = title
        self.description = description

    security.declareProtected(p_manageRSSCollection , 'aggregate')
    def aggregate(self, REQUEST=None):
        """ Now, let it runs with the feeds """

        #If aggregation has been disabled so ...
        if not getToolByName(self, 'portal_aggregation').isSiteAggregationAllowed():
            if REQUEST is not None:
                return REQUEST['RESPONSE'].redirect( self.absolute_url()
                                            + '/rsscollection_view'
                                            + '?portal_status_message=Aggregation+has+been+disabled'
                                            )
            else: 
                return "Aggregation has been disabled"

        feeds = self.objectValues('RSSFeed')
        for feed in feeds:
            if feed.status==STATUS_DEAD: 
                logger(0, 'Bypass %s because it was dead!' %feed.getId())
                continue #bypass dead feed if u do not want, please reset it's status
            feed.aggregate()
        logger(0, 'Aggregation Complete')

        if REQUEST is not None:
            return REQUEST['RESPONSE'].redirect( self.absolute_url()
                                        + '/rsscollection_view'
                                        + '?portal_status_message=Collection+updated'
                                        )

    security.declareProtected(p_manageRSSCollection , 'importFeeds')
    def importFeeds(self, feeds):
        """  """
        for feed in feeds:
            title = '%(sitename)s (%(siteurl)s, %(dataurl)s)' %feed
            dataurl = feed['dataurl']
            description = feed['description']
            fetchable = feed['fetchable']
            status = feed['status']
            categories = feed['Categories']

            id = b2a_base64(md5(dataurl).digest()).strip()
            id = id.translate(base64_to_id_trans, delete_chars)

            while id and (id[0] == '_' or id[0] == '-'):
                id=id[1:]
            while id and (id[-1] == '_' or id[-1] == '-'):
                id=id[:-1]

            if not hasattr(self.aq_explicit, id):
                self.invokeFactory( type_name='RSSFeed', 
                                    id =id, 
                                    title=title, 
                                    description=description, 
                                    dataurl=dataurl, 
                                    fetchable=fetchable, 
                                    status=status, 
                                    categories=categories)

InitializeClass(RSSCollection)
