from Globals import InitializeClass
from DateTime import DateTime
from AccessControl import ClassSecurityInfo
from Products.CMFDefault.Link import Link
from Products.CMFDefault.SkinnedFolder import SkinnedFolder
import urlparse
from Permissions import p_manageRSSFeed, p_View

from md5 import md5
from binascii import b2a_base64
from rssparser import parse
from AggregationTool import base64_to_id_trans, delete_chars

#For debugging
from logger import logger

from AggregationTool import STATUS_OK

factory_type_information = ( { 'id'             : 'RSSFeed'
                             , 'meta_type'      : 'RSSFeed'
                             , 'description'    : """ """
                             , 'icon'           : 'rssfeed_icon.gif'
                             , 'product'        : 'CMFAggregator'
                             , 'factory'        : 'addRSSFeed'
                             , 'immediate_view' : 'metadata_edit_form'
                             , 'allowed_content_types':['RSSStory']
                             , 'actions'        :
                                ( { 'id'            : 'view'
                                  , 'name'          : 'View'
                                  , 'action'        : 'rssfeed_view'
                                  , 'permissions'   : (p_View, )
                                  }
                                , { 'id'            : 'folder_contents'
                                  , 'name'          : 'Contents'
                                  , 'action'        : 'folder_contents'
                                  , 'permissions'   : (p_manageRSSFeed, )
                                  }
                                , { 'id'            : 'edit'
                                  , 'name'          : 'Edit'
                                  , 'action'        : 'portal_form/rssfeed_edit_form'
                                  , 'permissions'   : (p_manageRSSFeed, )
                                  }
                                , { 'id'            : 'metadata'
                                  , 'name'          : 'Metadata'
                                  , 'action'        : 'metadata_edit_form'
                                  , 'permissions'   : (p_manageRSSFeed, )
                                  }
                                )
                             }
                           ,
                           )

def addRSSFeed( self
              , id
              , title=''
              , description=''
              , dataurl=''
              , fetchable= 1
              , status=STATUS_OK 
              , categories = []
              ):
    """
        Add a RSSFeed instance to 'self'.
    """
    o=RSSFeed( id, title, description, dataurl, fetchable, status, categories)
    self._setObject(id,o)

###################################################
#Provide new Portal Type called RSSFeed 
class RSSFeed(SkinnedFolder, Link):
    """
        A RSS Feed
    """

    __implements__ = ( Link.__implements__, )

    meta_type = 'RSSFeed'

    security = ClassSecurityInfo()

    def __init__( self
                , id
                , title=''
                , description=''
                , dataurl=''
                , fetchable=1
                , status=STATUS_OK
                , categories=[]
                , last_updated=None
                , archivable=0
                ):
        """ New RSSFeed created """

        self.id=id
        self.title=title
        self.description=description
        self._editDataURL(dataurl)
        self.fetchable = 1
        self.status = status
        self.categories = categories
        self.last_updated = last_updated
        self.archivable = archivable
        self.format = 'text/url'

    security.declareProtected(p_manageRSSFeed, 'edit' )
    def edit(self,title='', description='', dataurl='', fetchable=1, status=STATUS_OK, categories=[], last_updated=None, archivable=0):
        """ Update and reindex. """
        self.title=title
        self.description=description
        self._editDataURL(dataurl)
        self.fetchable = 1
        self.status = status
        self.categories = categories
        if last_updated:
            self.editLastUpdated(last_updated)
        self.archivable = archivable

    security.declareProtected(p_manageRSSFeed, 'editLastUpdated' )
    def editLastUpdated(self, last_updated):
        if not isinstance(last_updated, DateTime):
            last_updated = DateTime(last_updated)
        self.last_updated = last_updated

    security.declarePrivate( '_editDataURL' )
    def _editDataURL( self, dataurl):
        """
            Edit the dataurl of the RSSFeed
        """
        tokens = urlparse.urlparse( dataurl, 'http' )
        if tokens[0] == 'http':
            if tokens[1]:
                # We have a nethost. All is well.
                url = urlparse.urlunparse(tokens)
            elif tokens[2:] == ('', '', '', ''):
                # Empty URL
                url = ''
            else:
                # Relative URL, keep it that way, without http:
                tokens = ('', '') + tokens[2:]
                url = urlparse.urlunparse(tokens)
        else:
            # Other scheme, keep original
            url = urlparse.urlunparse(tokens)
        self.dataurl = url

    security.declarePublic('SearchableText')
    def SearchableText(self):
        """
            All text for indexing including dataurl
        """
        return "%s %s %s" % (self.title, self.description, self.dataurl)

    security.declarePrivate('syncStory')
    def syncStory(self, id, fields):
        """ """
        pass

    security.declarePrivate('newStory')
    def newStory(self, id, fields):
        """ """
        (title, description, link) = fields
        self.invokeFactory(type_name='RSSStory',
                          id=id, 
                          title=title, 
                          description=description,
                          link=link
                          )

    security.declarePrivate('getStoryId')
    def getStoryId(self, fields):
        """ """
        id = b2a_base64(md5(''.join(fields)).digest()).strip()
        id = id.translate(base64_to_id_trans, delete_chars)
        while id and (id[0] == '_' or id[0] == '-'):
            id=id[1:]
        while id and (id[-1] == '_' or id[-1] == '-'):
            id=id[:-1]
        if not id:
            id = 'error_id'
        return id

    security.declareProtected(p_manageRSSFeed, 'aggregate')
    def aggregate(self, REQUEST=None):
        """ Aggregate news headlines using dataurl """
        dataurl = self.dataurl
        
        try:
            logger(0, 'Try to parse url: %s' %dataurl)
            results= parse(dataurl)
            logger(0, 'results: %d' % len(results['items']))
        except:
            logger(0, 'Fail')
            results = {'items':[]}
        
        new_ids = []
        for item in results['items']:
            fields = item.get('title', 'unknown'), item.get('description',''), item.get('link','')
            id = self.getStoryId(fields=fields)
            new_ids.append(id)
            if hasattr(self.aq_explicit, id):
                self.syncStory(id, fields=fields)
            else:
                self.newStory(id, fields=fields)

        if not self.archivable:
            old_ids = [id for id in self.objectIds() if id not in new_ids]
            logger(0, 'Delete old ids: %s' %`old_ids`)
            self.manage_delObjects(old_ids)

        if self.last_updated < DateTime():
            self.editLastUpdated(DateTime())
    
        if REQUEST is not None:
            return REQUEST['RESPONSE'].redirect( self.absolute_url()
                                        + '/rssfeed_view'
                                        + '?portal_status_message=Feed+reloaded'
                                        )


InitializeClass( RSSFeed )