import xmlrpclib
import httplib
from urllib2 import getproxies
from urlparse import urlparse

#For debugging
import logger

#########################################################
#Class for supporting proxy
class ProxyTransport(xmlrpclib.Transport):
    def __init__(self, http_proxy=""):
        self.http_proxy = http_proxy
        if not http_proxy:
            http_proxy = getproxies().get('http')
            if http_proxy:
                self.http_proxy = urlparse(http_proxy)[1]
            else:
                pass #No proxy found!
    def make_connection(self, host):
        if not self.http_proxy:
            return httplib.HTTP(host)
        else:
            self.real_host = host
            return httplib.HTTP(self.http_proxy)
    def send_request(self, connection, handler, request_body):
        if not self.http_proxy:
            connection.putrequest('POST', handler)
        else:
            connection.putrequest('POST', 'http://' + self.real_host + handler)

#########################################################
#Class for supporting Syndic8 services
class Syndic8Server:
    """
       This class exposes many services from Syndic8 server to AggregationTool.
    """
    def __init__(self, use_proxy=0, http_proxy=""):
        try:
            logger.logger(0, 'Initialize Syndic8 Server')
            if not use_proxy:
                self.Server = xmlrpclib.ServerProxy('http://www.syndic8.com/xmlrpc.php').syndic8
                logger.logger(0, 'Syndic8 Server with none proxy support')
            else:
                self.Server = xmlrpclib.ServerProxy('http://www.syndic8.com/xmlrpc.php', transport=ProxyTransport(http_proxy=http_proxy)).syndic8
                logger.logger(0, 'Syndic8 Server with proxy support, http_proxy: %s' %http_proxy)
        except:
            logger.logger(0, 'Fail to initialize Syndic8 Server')

    def getCategorySchemes(self):
        """ Get all available schemes from Syndic8  """
        try:
            logger.logger(0, 'Try to get schemes from Syndic8')
            schemes = self.Server.GetCategorySchemes()
            logger.logger(0, 'Successful!')
            return schemes
        except:
            logger.logger(0, 'Fail to get schemes from Syndic8')
            return []

    def getCategoryTree(self, scheme):
        """ Get all categories in a scheme """
        try:
            logger.logger(0, 'Try to get category tree for scheme %s' %scheme)
            categories = self.Server.GetCategoryTree(scheme)
            logger.logger(0, 'Successful!')
            return categories
        except:
            logger.logger(0, 'Fail to get category tree for scheme %s' %scheme)
            return []

    def getCategoryRoots(self, scheme):
        """ Get all root categories in a scheme """
        try:
            logger.logger(0, 'Try to get roots for scheme %s' %scheme)
            categories = self.Server.GetCategoryRoots(scheme)
            logger.logger(0, 'Successful!')
            return categories
        except:
            logger.logger(0, 'Fail to get roots for scheme %s' %scheme)
            return []

    def getFeedsInCategory(self, scheme, category):
        """ Get rss feed ids from (scheme, category) """
        try:
            logger.logger(0, 'Try to get feeds for scheme %s, category %s' %(scheme, category))
            feedids = self.Server.GetFeedsInCategory(scheme, category)
            logger.logger(0, 'Successful!')
            return feedids
        except:
            logger.logger(0, 'Fail to get feeds for scheme %s, category %s' %(scheme, category))
            return []

    def findFeeds(self, pattern):
        """ """
        try:
            logger.logger(0,'Start to search with pattern: %s' %pattern)
            ids = self.Server.FindFeeds(pattern)
            logger.logger(0, 'Results: %s' %str(ids))
            return ids
        except:
            logger.logger(0, 'Fail')
            return []

    def search(self, pattern="", scheme_map=[], fields=['feedid']):
        """ """
        schemes_results=[]
        pattern_results=[]
        last_results=[]

        if scheme_map:
            temp = {}
            for scheme in scheme_map.keys():
                categories = scheme_map[scheme]
                for category in categories:
                    ids = self.getFeedsInCategory(scheme, category)
                    for id in ids:
                        temp[id] = 'Appended' 
            if not temp: 
                return []
            else:
                schemes_results = temp.keys()

        if pattern:
            pattern_results = self.findFeeds(pattern)
            if not pattern_results: 
                return []

        if scheme_map and pattern:
            last_results = [x for x in schemes_results if x in pattern_results]
        elif scheme_map:
            last_results = schemes_results
        elif pattern:
            last_results = pattern_results

        feeds = self.getFeeds(last_results, fields)
        return feeds

    def getFeeds(self, ids, fields):
        """ """
        try:
            logger.logger(0, 'Try to get feeds for ids %s' %str(ids))
            feeds = self.Server.GetFeedInfo(ids, fields)
            logger.logger(0, 'Successful!')
            return feeds
        except:
            logger.logger(300, 'Fail')
            return []

#Test codes #####################################################
if __name__ == '__main__':
    logger.debug = 0
    tests = [5]

    if 1 in tests:
        #Test xmlrpclib with proxy support
        try:
            server = xmlrpclib.ServerProxy('http://www.oreillynet.com/meerkat/xml-rpc/server.php', transport=ProxyTransport())
            if not server.system.listMethods(): raise
            print "1 - OK"
        except:
            print "1 - Fail"

    if 2 in tests:
        #Test Syndic8Server with no proxy support
        try:
            server = Syndic8Server()
            if not server.getCategorySchemes(): raise
            print "2 - OK"
        except:
            print "2 - Fail"

    if 3 in tests:
        #Test Syndic8Server with proxy support
        try:
            server = Syndic8Server(use_proxy=1, http_proxy='192.168.50.1:8080')
            if not server.getCategorySchemes(): raise
            print server.search(pattern='cnn')
            print "3 - OK"
        except:
            print "3 - Fail"

    if 4 in tests:
        #Test Syndic8Server with auto detect proxy
        try:
            server = Syndic8Server(use_proxy=1, http_proxy="")
            if not server.getCategorySchemes(): raise
            print server.search(pattern='cnn')
            print "4 - OK"
        except:
            print "4 - Fail"

    if 5 in tests:
        #Test find feed
        try:
            server = Syndic8Server(use_proxy=1, http_proxy="192.168.50.1:8080")
            if not server.getCategorySchemes(): raise
            ids = server.search(pattern='security')
            feeds = self.Server.GetFeedInfo(ids, ['title'])
            for feed in feeds[:5]:
                print `feed`
            print "5 - OK"
        except:
            print "5 - Fail"
